<?php

/*
 * Plugin: Auto Team Adjustment
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * For a detailed description and documentation, please refer to:
 * http://labs.undef.de/XAseco1+2/Auto-Team-Adjustment.php
 *
 * ----------------------------------------------------------------------------------
 * Author:		undef.de
 * Version:		0.9.4
 * Date:		2012-02-19
 * Copyright:		2010 - 2012 by undef.de
 * System:		XAseco/1.06+ or XAseco2/0.90+
 * Game:		Trackmania Forever (TMF) or Trackmania2 (ManiaPlanet)
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies: none
 */

Aseco::registerEvent('onSync',				'ata_onSync');
Aseco::registerEvent('onPlayerConnect',			'ata_onPlayerConnect');

global $ata_config;

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function ata_onSync ($aseco) {
	global $ata_config;


	$xaseco1_min_version = '1.06';
	$xaseco2_min_version = '0.90';
	if (defined('XASECO_VERSION') && version_compare(XASECO_VERSION, $xaseco1_min_version, '<') ) {
		trigger_error('[plugin.auto_team_adjustment.php] Not supported XAseco version ('. XASECO_VERSION .')! Please update to min. version '. $xaseco1_min_version .'!', E_USER_ERROR);
	}
	else if (defined('XASECO2_VERSION') && version_compare(XASECO2_VERSION, $xaseco2_min_version, '<') ) {
		trigger_error('[plugin.auto_team_adjustment.php] Not supported XAseco2 version ('. XASECO2_VERSION .')! Please update to min. version '. $xaseco2_min_version .'!', E_USER_ERROR);
	}
	else if ( !defined('XASECO_VERSION') && !defined('XASECO2_VERSION') ) {
		trigger_error('[plugin.auto_team_adjustment.php] Can not identify the System, "XASECO_VERSION" or "XASECO2_VERSION" is unset! This plugin runs only with XAseco/'. $xaseco1_min_version .'+ or XAseco2/'. $xaseco2_min_version .'+', E_USER_ERROR);
	}


	// Setup Teammode ID
	if ( defined('XASECO_VERSION') ) {
		$ata_config['TEAMMODE'] = 2;
	}
	else if ( defined('XASECO2_VERSION') ) {
		$ata_config['TEAMMODE'] = 3;
	}


	$ata_config['PLAYERS'] = array();
	$ata_config['VERSION'] = '0.9.4';

	// Register this to the global version pool (for up-to-date checks)
	$aseco->plugin_versions[] = array(
		'plugin'	=> 'plugin.auto_team_adjustment.php',
		'author'	=> 'undef.de',
		'version'	=> $ata_config['VERSION']
	);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerConnect
function ata_onPlayerConnect ($aseco, $player) {
	global $ata_config;


	// Bail out immediately if not Team-Mode
	if ($aseco->server->gameinfo->mode != $ata_config['TEAMMODE']) {
		return;
	}


	// Get the detailed Player informations
	$aseco->client->query('GetDetailedPlayerInfo', $player->login);
	$info = $aseco->client->getResponse();

	// Store TeamId of Player
	$ata_config['PLAYERS'][$player->login]['team'] = $info['TeamId'];

	// Calculate Player weighting
	if ($info['LadderStats']['NbrMatchDraws'] > 0) {
		$player_drawn = $info['LadderStats']['NbrMatchDraws'] / 2;
	}
	$player_win = $info['LadderStats']['NbrMatchWins'] + $player_drawn;
	$player_lost = $info['LadderStats']['NbrMatchLosses'] + $player_drawn;
	if ( ($player_win) && ($player_lost) ) {
		$ata_config['PLAYERS'][$player->login]['weighting'] = round( ($player_win / $player_lost), 2);
	}
	else {
		$ata_config['PLAYERS'][$player->login]['weighting'] = 0;
	}

//	$aseco->console('CONF Player: '. $player->login .' Weighting: '. $ata_config['PLAYERS'][$player->login]['weighting'] .' Team: '. $ata_config['PLAYERS'][$player->login]['team']);


	if (count($aseco->server->players->player_list) > 1) {

		// Blue Team (Id: 0)
		$blue_weighting = 0;
		$blue_playercount = 0;

		// Red Team (Id: 1)
		$red_weighting = 0;
		$red_playercount = 0;

		for ($team = 0; $team <= 1; $team++) {
			foreach ($ata_config['PLAYERS'] as $login => $struct) {

				if ( ($ata_config['PLAYERS'][$login]['team'] == $team) && ($team == 0) ) {
					// Blue Team
					$blue_weighting += $ata_config['PLAYERS'][$login]['weighting'];
					$blue_playercount ++;

				}
				else if ( ($ata_config['PLAYERS'][$login]['team'] == $team) && ($team == 1) ) {
					// Red Team
					$red_weighting += $ata_config['PLAYERS'][$login]['weighting'];
					$red_playercount ++;
				}

			}
		}
		unset($team);


		$team = false;
		if ( ($blue_weighting > $red_weighting) && ($blue_playercount > $red_playercount) ) {
			$team = 1;		// Force to Red Team
		}
		else if ( ($blue_weighting < $red_weighting) && ($blue_playercount < $red_playercount) ) {
			$team = 0;		// Force to Blue Team
		}
		else if ($blue_playercount > $red_playercount) {
			$team = 1;		// Force to Red Team
		}
		else if ($blue_playercount < $red_playercount) {
			$team = 0;		// Force to Blue Team
		}


//		$aseco->console('[plugin.auto_team_adjustment.php] Blue: '. $blue_weighting .', '. $blue_playercount .' Players; Red: '. $red_weighting .', '. $red_playercount .' Players; Force Team: '. (($team === false) ? bool2text($team) : $team));


		if ($team !== false) {

			// Save changed Team
			$ata_config['PLAYERS'][$player->login]['team'] = $team;

			$message = '{#server}>> Force Player '. $player->nickname .'$Z$S{#server} into the '. (($team == 0) ? '$19FBlue' : '$F50Red') .' Team';
			if ( function_exists('send_window_message') ) {
				send_window_message($aseco, $message, false);
			}
			else {
				$aseco->client->query('ChatSendServerMessage', $aseco->formatColors($message));
			}

			// Force the Player to an Team
			$aseco->client->query('ForcePlayerTeam', $player->login, $team);
		}
	}
}

?>
